from katka.constants import STATUS_INACTIVE
from katka.fields import username_on_model
from rest_framework import status, viewsets
from rest_framework.response import Response


class AuditViewSet(viewsets.ModelViewSet):
    model = None

    def get_queryset(self):
        return self.model.objects.exclude(status=STATUS_INACTIVE)

    def create(self, request, *args, **kwargs):
        with username_on_model(self.model, request.user.username):
            return super().create(request, *args, **kwargs)

    def update(self, request, *args, **kwargs):
        with username_on_model(self.model, request.user.username):
            return super().update(request, *args, **kwargs)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        instance.status = STATUS_INACTIVE

        with username_on_model(self.model, request.user.username):
            instance.save()

        return Response(status=status.HTTP_204_NO_CONTENT)



from katka.models import Team
from katka.serializers import TeamSerializer
from katka.viewsets import AuditViewSet


class TeamViewSet(AuditViewSet):
    model = Team
    serializer_class = TeamSerializer
    lookup_field = 'public_identifier'
    lookup_value_regex = '[0-9a-f-]{36}'

    def get_queryset(self):
        # Only show teams that are linked to a group that the user is part of
        user_groups = self.request.user.groups.all()
        return super().get_queryset().filter(group__in=user_groups)



from django.contrib import admin
from django.urls import include, path

from rest_framework import routers

from . import views

app_name = 'katka_core'

router = routers.SimpleRouter()
router.register('team', views.TeamViewSet, basename='team')

urlpatterns = [
    path('admin/', admin.site.urls),
    path('', include(router.urls)),
]



from django.contrib import admin

from katka.fields import username_on_model
from katka.models import Team


@admin.register(Team)
class TeamAdmin(admin.ModelAdmin):
    fields = ('name', 'group')

    def save_model(self, request, obj, form, change):
        with username_on_model(self.model, request.user.username):
            super().save_model(request, obj, form, change)



from django.contrib.auth.models import Group

from katka.models import Team
from rest_framework import serializers
from rest_framework.exceptions import NotFound, PermissionDenied


class GroupNameField(serializers.RelatedField):
    def to_internal_value(self, group_name):
        qs = self.get_queryset()
        try:
            return qs.get(name=group_name)
        except qs.model.DoesNotExist:
            raise NotFound

    def to_representation(self, group):
        return group.name


class TeamSerializer(serializers.ModelSerializer):
    group = GroupNameField(queryset=Group.objects.all())

    class Meta:
        model = Team
        fields = ('public_identifier', 'name', 'group')

    def validate_group(self, group):
        if not self.context['request'].user.groups.filter(name=group.name).exists():
            raise PermissionDenied('User is not a member of this group')

        return group



import uuid

from django.contrib.auth.models import Group
from django.db import models

from katka.auditedmodel import AuditedModel


class Team(AuditedModel):
    public_identifier = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=100)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)

    def __str__(self):
        return f'{self.group.name}'
